#ifndef HTTP_H
#define HTTP_H
#include <string>
#include <vector>
#include <map>
#include "socket.h"

namespace Net
{
std::string urlencode(const std::string &str)
{
	std::string s;
	for (std::string::const_iterator it = str.begin();it!=str.end();++it) {
		char c = *it;
		char h[8];
		if (c>='A' && c<='Z' || c>='a' && c<='z' || c>='0' && c<='9') {
			s+=c;
		} else {
			sprintf(h,"%%%02x",c);
			s+=h;
		}
	}
	return s;
}
std::string urldecode(const std::string &str)
{
	std::string s;
	for (std::string::const_iterator it = str.begin();it!=str.end();++it) {
		char c = *it;
		char h[8];
		if (c!='%') {
			s+=c;
		} else {
			// ŘAᖳmȂ̂ŃRs[
			h[0]=*++it;
			h[1]=*++it;
			h[2]=0;
			int d;
			sscanf(h,"%02x",&d);
			s+=(char)d;
		}
	}
	return s;
}


class HttpClient
{
public:
	enum METHOD {
		AUTO,
		GET,
		POST,
		HEAD,
	};
	METHOD method;
	std::string body;
	std::map<std::string, std::vector<std::string> > headers;
	std::map<std::string,std::string> header;
	std::map<std::string,std::string> req_header;
	int status;

	HttpClient()
	{
		method = AUTO;
	}

	Socket request(const std::string &host, int port,const std::string &path ,const std::string &data="")
	{
		status=0;
		header.clear();

		if (!req_header.count("Host")) req_header["Host"] = host;
		std::string methodstr;
		if (method==AUTO) {
			methodstr=data.size()?"POST":"GET";
		} else {
			methodstr=(method==POST)?"POST":"GET";
		}
		if (method==POST || method==AUTO&&data.size()) {
			char s[10];
			sprintf(s,"%d", data.size());
			req_header["Content-Length"] = s;
		}
		req_header["Connection"] = "close";

		Socket soc(host,port);
		if (method==GET && data.size()) {
			soc.write(methodstr+" "+path+"?"+data+" HTTP/1.1\n");
		} else {
			soc.write(methodstr+" "+path+" HTTP/1.1\n");
		}
		for (std::map<std::string,std::string>::iterator it=req_header.begin();it!=req_header.end();++it) {
			soc.write(it->first+": "+it->second+"\n");
		}
		soc.write("\n" );
		if (soc.error()) return soc;

		if (method!=GET && data.size()) {
			soc.write(data);
		}

		std::string line;
		line = soc.readLine(); // HTTP status
		int p=line.find(" ");
		if (p!=std::string::npos) {
			status = atoi(line.c_str()+p+1);
		}
		while(!soc.error()) {
			line = soc.readLine();
#ifdef DEBUG
			std::cout << line << std::endl;
#endif
			if (line.size()==0) break;
			if (line.size() && line[line.size()-1]=='\r') line.resize(line.size()-1);
			if (line.empty()) break;
			int p=line.find(":");
			std::string name=line.substr(0, p);
			if (line[p+1]==' ') p++;
			std::string value=line.substr(p+1);
			header[name]=value;
			headers[name].push_back(value);
			//cout << name << " : " << value << endl;
		}
		return soc;
	}
	
	Socket request(const std::string &url, const std::string &data="")
	{
		using namespace std;
		int s=0;
		if (url.substr(0,7)=="http://") s=7;
		int p=url.find("/",s);
		string host = url.substr(s, p-s);
		string path = url.substr(p);
		int port=80;

		p=host.find(":");
		if (p!=string::npos) {
			port=atoi(host.substr(p+1).c_str());
			host = host.substr(0, p);
		}
		return request(host, port, path, data);
	}

	int load(const std::string &url, const std::string &data="")
	{
		Socket soc = request(url, data);
		while(!soc.error()) {
			body += soc.read();
		}
		soc.close();
		return body.size();
	}
	
	void clear()
	{
		body.clear();
		header.clear();
		headers.clear();
		method = AUTO;
		req_header.clear();
	}

	int load(const std::string &url, const std::map<std::string,std::string> &params)
	{
		std::string data;
		for (std::map<std::string,std::string>::const_iterator it = params.begin();it!=params.end();++it) {
			if (data!="") data+="&";
			data += (*it).first;
			data += '=';
			data+= urlencode((*it).second);
		}
		return load(url,data);
	}

};

}

#endif

