#include <windows.h>
#include <string>
#include <complex>
#include <vector>

#define NOIMP
#include "cppfl/all.h"

#include "labeling.h"

using namespace std;
using namespace kwlib;

#include "detect.h"

#pragma warning(disable:4244) // double -> int

#ifndef PI
#    define PI (3.1415926535897932384626433832795)
#endif
// outer product
static double cross(P a,P b) {
	return a.real()*b.imag() - a.imag()*b.real();
}
static double dot(P a,P b) {
	return a.real()*b.real() + a.imag()*b.imag();
}
static P nv(P a){
	return a/abs(a);
}
static P intersection_l(P a1, P a2, P b1, P b2) {
	P a = a2 - a1; P b = b2 - b1;
	return a1 + a * cross(b, b1-a1) / cross(b, a);
}

template<int N,typename T>
void arrayrotN(T p[N],int d)
{
	T t[N];
	int i;
	for (i=0;i<d;i++) t[i]=p[i];
	for (i=d;i<N;i++) p[i-d]=p[i];
	for (i=0;i<d;i++) p[4-d+i]=t[i];
}


static bool findgap(const Bitmap &img,const P &p1,const P &p2,double len)
{

	P st(p1.real(),p1.imag());
	for(line_iterator it(p1.real(),p1.imag() , p2.real() ,p2.imag());it;++it ) {
		if (img.get(it.x,it.y)==0) {
			if (abs(st-P(it.x,it.y))>len) return true;
		} else {
			st=P(it.x,it.y);
		}
	}
	return false;
}


/*

void stretch(Bitmap &dst,Bitmap &img,const vector<P> vert) {
	int x,y;
	for (y=0;y<dst.height;y++) {
		P t1=(vert[0]*(double)(255-y)+vert[3]*(double)y) / 255.0;
		P t2=(vert[1]*(double)(255-y)+vert[2]*(double)y) / 255.0;
		t1.real(t1.real()+2);
		for (x=0;x<dst.width;x++) {
			P p=(t1*(double)(255-x)+t2*(double)x) / 255.0;
			dst.pixel(x,y)->rgb=img.pixel(p.real(),p.imag())->rgb;
		}
	}
}

*/

static double avg(const Bitmap &img,const Point &p1,const Point &p2)
{
	double sum=0.0;
	int count=0;
	for(line_iterator it(p1.x,p1.y , p2.x ,p2.y);it;++it ) {
		sum+=img.pixel(it.x,it.y)->r;
		count++;
	}
	return count?sum/count:0.0;
}

void thresholdRGB(Bitmap &dst,const Bitmap &src,int th){
	for (scan_iterator it(src);it;++it) {
		dst.set(it.x,it.y,(*it).getv()<th?1:0);
	}
}

// adjust
int adjust( Bitmap &src,P c0[4],P c1[4],P c2[4],P &p3)
{
	//c0[0]+=nv(c0[0]-c0[2])*1.0;
	//c1[0]+=nv(c1[0]-c1[2])*1.0;
	//c2[0]+=(c2[0]-c2[2])/20.0;

	// Adjust p3
	int i;

	int oa,a;
	P v;
	v = nv(c2[1]-c2[0]);
	P p3a=p3+v*10.0;
	oa=0;
	for (i=0;i<21;i++) {
		a=avg(src,(c1[0]*2.0+p3a)/3.0+v*4.0,p3a);
		if (a>3) break;
		oa=a;
		p3a-=v;
	}

	v = nv(c1[3]-c1[0]);
	P p3b=p3+v*10.0;
	oa=0;
	for (i=0;i<21;i++) {
		a=avg(src,(c2[0]*2.0+p3b)/3.0+v*4.0,p3b);
		if (a>3) break;
		oa=a;
		p3b-=v;
	}
	p3 = intersection_l(c1[0],p3a,c2[0],p3b);

	// get code size
	int mode = 1;
	int d,od=1;
	int count=0;

	P p1=(c0[2]*13.0+c0[3])/14.0;
	P p2=(c1[2]*13.0+c1[1])/14.0;
	for(line_iterator it(p1.real(),p1.imag() , p2.real() ,p2.imag());it;++it ) {
		d= ( src.pixel(it.x,it.y)->r!=0 );
		if (d==od) {
			if (d!=mode){
				count++;
				mode = d;
			}
		}
		od = d;
	}

	if (mode) count--;

	return count+14;
}

void postalize(Bitmap &dst,const Bitmap &img,P c0[4],P c1[4],P c2[4],P c3,int sz) {
	//c3.real(c3.real()-7.0);
	P vert[]={c0[0],c1[0],c3,c2[0]};
	int x,y;
	double ax,dx;
	P dy1,dy2;
	P ay1,ay2;
	{
		double y1=abs(c0[1]-c0[0])/abs(vert[3]-vert[0]) , x1=7.0/sz;
		double a1=(y1-x1)/((x1-1)*x1);
		double y2=abs(c2[3]-c2[0])/abs(vert[3]-vert[0]) , x2=7.0/sz;
		double a2=(y2-x2)/((x2-1)*x2);
		a1=(a1-a2)/2.0; // 

		double ac=a1*2/(255);
		double v=(1-a1);
		dy1=(vert[3]-vert[0])*(1-a1)/255.0;
		ay1=(vert[3]-vert[0])*(a1/128/255);
		dy2=(vert[2]-vert[1])*(1-a1)/255.0;
		ay2=(vert[2]-vert[1])*(a1/128/255);
	}
	{
		double y1=abs(c0[3]-c0[0])/abs(vert[1]-vert[0]) , x1=7.0/(sz);
		double a1=(y1-x1)/((x1-1)*x1);
		double y2=abs(c1[0]-c1[1])/abs(vert[1]-vert[0]) , x2=x1;
		double a2=(y2-x2)/((x2-1)*x2);
		a1=(a1-a2)/2.0; // 
		ax=a1*2/(255);
		dx=(1-a1);
	}
	P y1=vert[0];
	P y2=vert[1];
	for (y=0;y<dst.height;y++) {
		P p=y1;
		P dx1=dx*(y2-y1)/255.0;
		P ax1=ax*(y2-y1)/255.0;
		for (x=0;x<dst.width;x++) {
			dst.pixel(x,y)->rgb=img.pixel(p.real(),p.imag())->rgb;
			p+=dx1;
			dx1+=ax1;
		}
		y1+=dy1;
		y2+=dy2;
		dy1+=ay1;
		dy2+=ay2;
	}
}


int DetectQR::detect( Bitmap &img){
		srcimg=&img;
		count=0;
		pat.clear();
		tmp = &img;

		LabelingSS labeling;
		thresholdRGB(lbuf,img,120);
		labeling.Exec((short*)lbuf.buf, (short*)lbuf.buf, img.width, img.height, false, 10);
/*
		tmp->clear();
		for (scan_iterator it(*tmp);it; ++it ) {
			(*it).rgb=RGB24(lbuf(it.x,it.y)?255:0,0,0);
		}
*/
		int x;
		int i,j;
		// ʒuop^[
		float cx,cy;
		int n=labeling.GetNumOfResultRegions();
		int mm=0;
		for (i=0;i<n;i++) {
			labeling.GetResultRegionInfo(i)->GetCenter(cx,cy);
			int c=lbuf(cx,cy)-1;

			if (c<0 || c==i)continue;
			int sx,sy,mx,my;
			labeling.GetResultRegionInfo(i)->GetMin(sx,sy);
			labeling.GetResultRegionInfo(i)->GetMax(mx,my);
			if (sx==0 || sy==0 || mx>=img.width-1 || my>=img.height-1) continue;
			int s1=labeling.GetResultRegionInfo(i)->GetNumOfPixels();
			int s2=labeling.GetResultRegionInfo(c)->GetNumOfPixels();
			if (s1*9<s2*32 && s1*9>s2*16 ) { // 9:24
				img.set(cx,cy,0xffff);
				for (x=sx;x<=mx;x++) {
					if (lbuf(x,sy)!=i+1) continue;
					vector<Point> v;
					for (edge_iteratorS it(lbuf,x,sy);it;++it ) {
						v.push_back(it);
						//cout << it.x << "," << it.y << endl;
					}

					int sz=v.size();
					if (sz<40) continue;
					const static int dl=7;
					double max=0,maxn=0,d,od=0;
					int edge_n[8];
					double edge_v[8];
					int count=0,state=0;

					// lp`̊p
					for (j=0;j<sz;j++) {
						img.set(v[j].x,v[j].y,0x00ff00);
						P p1=P(v[j].x,v[j].y);
						P p2=P(v[(j+dl)%sz].x,v[(j+dl)%sz].y);
						P p3=P(v[(j+dl*2)%sz].x,v[(j+dl*2)%sz].y);
						d=arg((p2-p1)/(p3-p2));
						if (d>max) {
							max=d; maxn=j+dl;
						}
						od=d;
						if (d<max/2) {
							state=1;
							if (max>0.4) {
								if (count<5) {
									// add
									edge_n[count]=maxn;
									edge_v[count]=max;
									//img.set(v[(edge_n[count])%sz].x,v[(edge_n[count])%sz].y,0xff00ff);
									count++;
									//cout << "deg:" << max << endl;
								}
								if (j+dl< maxn+sz/8) {
									j = maxn+sz/8;
								}
							}
							max=0;
						}
					}
					if (count==5) {
						// ŏƍŌ̂傫c
						if (edge_v[4] > edge_v[0]) {
							// shift
							for (int j=0;j<4;j++) {
								edge_n[j]=edge_n[j+1];
								edge_v[j]=edge_v[j+1];
							}
						}
						count=4;
					}
					//cout << endl;
					if (count==4) {
						for (j=0;j<count;j++) {
							//P c=intersection_l(v[(edge_n[(j+3)%4]+dl)%sz],v[(edge_n[j]-dl)%sz],v[(edge_n[j]+dl)%sz],v[(edge_n[(j+1)%4]-dl)%sz]);
							//PIXEL *p=tmp->pixel(c);
							//PIXEL *p=tmp->pixel(v[(edge_n[j])%sz]);
							//p->r=255;
							img.set(v[(edge_n[j])%sz].x,v[(edge_n[j])%sz].y,0xff00ff);
							cp[mm][j]=P(v[(edge_n[j])%sz].x,v[(edge_n[j])%sz].y);
						}
						mm++;
					}

					break;
				}
			}
		}
		ptn_count=mm;
		if (mm<3) return 0;


		DCPen *pen=(DCPen*)(*(DIBitmap*)&img).pen();
		//pen->color(COLOR::white);
		// ۂp^[gݍ킹
		char table[80]={0};
		//pen->color(COLOR::green);
		for (i=0;i<mm;i++) {
			if (table[i]) continue;
			int mode[2];
			int ff[2]={-1,-1};
			for (int l=0;l<2;l++) {
				for (j=0;j<mm;j++) {
					if (table[j]) continue;
					if (j==i) continue;
					for (int k=0;k<4;k++) {
						double er1 = abs(cross(nv(cp[i][l+1]-cp[i][l]),nv(cp[j][k]-cp[i][l])))
									+abs(cross(nv(cp[i][l+1]-cp[i][l]),nv(cp[j][(k+1)%4]-cp[i][l])));
						double er2 = abs(cross(nv(cp[i][(l+3)%4]-cp[i][l+2]),nv(cp[j][(k+2)%4]-cp[i][l+2])))
									+abs(cross(nv(cp[i][(l+3)%4]-cp[i][l+2]),nv(cp[j][(k+3)%4]-cp[i][l+2])));
						if (er1<0.5 && er2<0.5 ) {
							if (findgap(*tmp,(cp[i][l]+cp[i][l+2])/2.0,(cp[j][k]+cp[j][(k+2)%4])/2.0,abs(cp[i][l]-cp[i][l+1]))) {
								continue;
							}

							mode[l]= dot(cp[i][l+1]-cp[i][l],cp[j][k]-cp[i][l])>0;
							ff[l]=j;
							break;
						}
					}
				}
				if (ff[0]<0) break;
			}
			if (ff[0]<0 || ff[1]<0) continue;
			table[i]=1;
			table[ff[0]]=1;
			table[ff[1]]=1;
			int p0=0;
			p0=(mode[0]|2)^(mode[1]*3);

			if (p0) arrayrotN<4>(cp[i],p0);
			pat.push_back(i);
			pat.push_back(ff[0]);
			pat.push_back(ff[1]);
		//	pen->circlef(cp[i][0].real(),cp[i][0].imag(),3);
			//img.set(cp[i][0].real(),cp[i][0].imag(),0x0000ff);
		}


		for (i=0;i<(int)pat.size();i+=3) {
			int n=pat[i];
			// p^[𑵂̏
			if (arg((cp[pat[i+1]][0]-cp[pat[i]][0])/(cp[pat[i]][0]-cp[pat[i+2]][0]))<0) {
				int t=pat[i+1]; pat[i+1]=pat[i+2]; pat[i+2]=t;
			}

			// p
			for (j=i+1;j<i+3;j++) {
				int k2=(j==i+1)?3:1;
				double me=10.0;
				int mk=0;
				int k;
				for (k=0;k<4;k++) {
					double er = abs(cross(nv(cp[n][k2]-cp[n][0]),nv(cp[pat[j]][k]-cp[n][0])))
								+abs(cross(nv(cp[n][k2]-cp[n][0]),nv(cp[pat[j]][(k+1)%4]-cp[n][0])));
					if (er<me) {me=er;mk=k;}
				}
				k=mk;
				if (abs(cp[pat[j]][k]-cp[pat[i]][0])<abs(cp[pat[j]][(k+1)%4]-cp[pat[i]][0])) {
					k=(k+1)%4;
				}
				arrayrotN<4>(cp[pat[j]],k);
			}

			img.set(cp[pat[i]][0].real(),cp[pat[i]][0].imag(),0x0000ff);

//			pen->color(COLOR::yellow);
//			pen->circlef(cp[pat[i+1]][0].real(),cp[pat[i+1]][0].imag(),3);
//			pen->color(COLOR::blue);
//			pen->circlef(cp[pat[i+2]][0].real(),cp[pat[i+2]][0].imag(),3);
//			pen->color(COLOR::green);
//			pen->circlef(cp[pat[i]][0].real(),cp[pat[i]][0].imag(),3);
		}

		int oc=pat.size()/3;
		if (oc<=0) return 0;
		if (oc>MAX_QRCODE_COUNT) oc=MAX_QRCODE_COUNT;

		for (i=0;i<oc;i++) {
			int c0=pat[i*3+0],c1=pat[i*3+1],c2=pat[i*3+2];
			p3[i] = intersection_l(cp[c1][0],cp[c1][3],cp[c2][0],cp[c2][1]);
			//cout << p3[i] << endl;
			//if (p3[i].real()<0 || p3[i].imag()<0) return -1;
			img.set(p3[i].real(),p3[i].imag(),0x0000ff);
			size[i]=adjust(lbuf,cp[c0],cp[c1],cp[c2],p3[i]);
		}

		count=oc;
		return oc;
}
void DetectQR::getQRImage(Bitmap &qr,int i) {
	int c0=pat[i*3+0],c1=pat[i*3+1],c2=pat[i*3+2];
	postalize(qr,*srcimg,cp[c0],cp[c1],cp[c2],p3[i],size[i]);
}

void DetectQR::getCode(Bitmap8 &code,int i) {
	int c0=pat[i*3+0],c1=pat[i*3+1],c2=pat[i*3+2];
	postalize(*qrimg,*srcimg,cp[c0],cp[c1],cp[c2],p3[i],size[i]);
	int sz=code.height;
	int x,y;


	int dx[300],dy[300];
	int nx[100],ny[100];
	int cntx=0,cnty=0;
	int o=0,n=0;
	int min,max=0;

	// x
	for (x=0;x<qrimg->width-1;x++) {
		int sum=0;
		for (y=0;y<qrimg->height;y++) {
			sum+=abs((*qrimg)(x,y).getv()-(*qrimg)(x+1,y).getv());
		}
		dx[x]=sum;
	}
	max=0;min=10000;o=0;
	int on=0;
	for (x=0;x<qrimg->width-2;x++) {
		int d=(dx[x]+dx[x+1]+dx[x+2])/3;
		if (d<min) min=d;
		if (d>max && d*7>min*10 && (x-o)*3>(256/sz)*2) {
			max=d;
			n=x+1;
		}
		if (d*10<max*7 ) {
			int steps=(n-o+128/sz)/(256/sz); // FIXME!
			for (int t=1;t<=steps;t++ ) {
				//dx[o+(n-o)*t/steps]=7500;
				nx[cntx++]=o+(n-o)*t/steps-on;
				on=o+(n-o)*t/steps;
			}
			//dx[n]=8000;
			min=max;
			max=0;
			o=n;
		}
	}
	nx[cntx++]=256-on;

	// y
	for (y=0;y<qrimg->height-1;y++) {
		int sum=0;
		for (x=0;x<qrimg->width;x++) {
			sum+=abs((*qrimg)(x,y).getv()-(*qrimg)(x,y+1).getv());
		}
		dy[y]=sum;
	}
	max=0;min=10000;o=0;on=0;
	for (y=0;y<qrimg->height-2;y++) {
		int d=(dy[y]+dy[y+1]+dy[y+2])/3;
		if (d<min) min=d;
		if (d>max && d*7>min*10 && (y-o)*3>(256/sz)*2) {
			max=d;
			n=y+1;
		}
		if (d*10<max*7 ) {
			int steps=(n-o+128/sz)/(256/sz); // FIXME!
			for (int t=1;t<=steps;t++ ) {
				//dy[o+(n-o)*t/steps]=7500;
				ny[cnty++]=o+(n-o)*t/steps-on;
				on=o+(n-o)*t/steps;
			}
			//dy[n]=8000;
			min=max;
			max=0;
			o=n;
		}
	}
	ny[cnty++]=256-on;

//	for (x=0;x<qrimg->width-1;x++) {
//		cout << dx[x] << endl;
//	}
//	cout << endl;
//	cout << cntx << "," << cnty<< endl;

	if (cntx<sz|| cnty<sz) return ;

	long s[100][100]={{0}};
	int xx,yy=0;
	for (y=0;y<sz;y++) {
		xx=0;
		for (x=0;x<sz;x++) {
			for (int i=0;i<ny[y];i++) {
				for (int j=0;j<nx[x];j++) {
					s[y][x]+=qrimg->pixel(xx+j,yy+i)->getv();
				}
			}
			s[y][x]/=ny[y]*nx[x];
			xx+=nx[x];
		}
		yy+=ny[y];
	}

	int wm=65535;
	int bm=0;
	for (int i=0;i<7;i++) {
		if (s[6][i]>bm) bm=s[6][i];
		if (s[7][i]<wm) wm=s[7][i];
	}
	int th=(wm+bm)/2.0;

	for (y=0;y<qrimg->height;y++) {
		for (x=0;x<qrimg->width;x++) {
			qrimg->pixel(x,y)->g=s[y*sz/qrimg->height][x*sz/qrimg->width]>th?255:0;
			if (dx[x]==7500 || dx[x]==8000)
				qrimg->pixel(x,y)->r=dx[x]==8000?255:0;
			if (dy[y]==7500 || dy[y]==8000)
				qrimg->pixel(x,y)->r=dy[y]==8000?255:0;
		}
	}
	for (y=0;y<sz;y++) {
		for (x=0;x<sz;x++) {
			code(x,y)=s[y][x]>th?1:0;
		}
	}

}


void DetectQR::getCode2(Bitmap8 &code,int i) {
	int c0=pat[i*3+0],c1=pat[i*3+1],c2=pat[i*3+2];
	postalize(*qrimg,*srcimg,cp[c0],cp[c1],cp[c2],p3[i],size[i]);
	int sz=code.height;
	int x,y;

	long s[100][100]={{0}};
	for (y=0;y<qrimg->height;y++) {
		for (x=0;x<qrimg->width;x++) {
			s[y*sz/qrimg->height][x*sz/qrimg->width]+=qrimg->pixel(x,y)->getv();
		}
	}

	int wm=65535;
	int bm=0;
	for (int i=0;i<7;i++) {
		if (s[6][i]>bm) bm=s[6][i];
		if (s[7][i]<wm) wm=s[7][i];
	}
	int th=(wm+bm)/2.0;

	for (y=0;y<qrimg->height;y++) {
		for (x=0;x<qrimg->width;x++) {
			qrimg->pixel(x,y)->g=s[y*sz/qrimg->height][x*sz/qrimg->width]>th?255:0;
		}
	}
	for (y=0;y<sz;y++) {
		for (x=0;x<sz;x++) {
			code(x,y)=s[y][x]>th?1:0;
		}
	}

}
